<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Classes;

    use Exception;
    use FederationLib\Classes\Configuration\CacheServerConfiguration;
    use FederationLib\Classes\Configuration\TamerLibConfiguration;
    use FederationLib\Enums\Standard\Methods;
    use FederationLib\Enums\Standard\PermissionRole;
    use RuntimeException;

    class Configuration
    {
        /**
         * @var \ConfigLib\Configuration|null
         */
        private static $configuration;

        /**
         * @var TamerLibConfiguration|null
         */
        private static $tamerlib_configuration;

        /**
         * Returns the full raw configuration array.
         *
         * @return array
         */
        public static function getConfiguration(): array
        {
            if(self::$configuration === null)
            {
                self::$configuration = new \ConfigLib\Configuration('federation');

                /** Database Configuration */
                self::$configuration->setDefault('database.driver', 'mysqli');
                self::$configuration->setDefault('database.host', 'localhost');
                self::$configuration->setDefault('database.port', 3306);
                self::$configuration->setDefault('database.name', 'federation');
                self::$configuration->setDefault('database.username', 'root');
                self::$configuration->setDefault('database.password', 'root');
                self::$configuration->setDefault('database.charset', 'utf8mb4');
                self::$configuration->setDefault('database.reconnect_interval', 1800);

                /** Multi-Cache Configuration */
                // Cache System Configuration
                self::$configuration->setDefault('cache_system.enabled', true);
                self::$configuration->setDefault('cache_system.opened_connection_priority', 20); // Higher is better
                self::$configuration->setDefault('cache_system.error_connection_priority', -30); // Lower is better
                // Client Objects
                self::$configuration->setDefault('cache_system.cache.client_objects_enabled', true);
                self::$configuration->setDefault('cache_system.cache.client_objects_ttl', 200);
                self::$configuration->setDefault('cache_system.cache.client_objects_server_preference', 'redis_master');
                self::$configuration->setDefault('cache_system.cache.client_objects_server_fallback', 'redis_slave');
                // PeerRecord Objects
                self::$configuration->setDefault('cache_system.cache.peer_objects_enabled', true);
                self::$configuration->setDefault('cache_system.cache.peer_objects_ttl', 200);
                self::$configuration->setDefault('cache_system.cache.peer_objects_server_preference', 'redis_master');
                self::$configuration->setDefault('cache_system.cache.peer_objects_server_fallback', 'redis_slave');
                /** Multi-Cache Server Configuration */
                // Redis Master Configuration
                self::$configuration->setDefault('cache_system.servers.redis_master.enabled', true);
                self::$configuration->setDefault('cache_system.servers.redis_master.host', 'localhost');
                self::$configuration->setDefault('cache_system.servers.redis_master.port', 6379);
                self::$configuration->setDefault('cache_system.servers.redis_master.priority', 100);
                self::$configuration->setDefault('cache_system.servers.redis_master.password', null);
                self::$configuration->setDefault('cache_system.servers.redis_master.database', null);
                self::$configuration->setDefault('cache_system.servers.redis_master.reconnect_interval', 1800);
                // Redis Slave Configuration
                self::$configuration->setDefault('cache_system.servers.redis_slave.enabled', false);
                self::$configuration->setDefault('cache_system.servers.redis_slave.host', 'localhost');
                self::$configuration->setDefault('cache_system.servers.redis_slave.port', 11211);
                self::$configuration->setDefault('cache_system.servers.redis_slave.priority', 50);
                self::$configuration->setDefault('cache_system.servers.redis_slave.password', null);
                self::$configuration->setDefault('cache_system.servers.redis_slave.database', null);
                self::$configuration->setDefault('cache_system.servers.redis_slave.reconnect_interval', 1800);

                /** Federation Configuration */
                self::$configuration->setDefault('federation.hostname', 'FederationLib'); // Server Hostname
                self::$configuration->setDefault('federation.events_retention', 1209600); // Two Weeks
                self::$configuration->setDefault('federation.security.strict_permissions', true); // Security Feature, prevents clients & peers from elevating their permissions
                self::$configuration->setDefault('federation.security.method_permissions.ping', 5); // Guest or above
                self::$configuration->setDefault('federation.security.method_permissions.whoami', 5); // Guest or above
                self::$configuration->setDefault('federation.security.method_permissions.create_client', 2); // Admin or above
                self::$configuration->setDefault('federation.security.method_permissions.get_client', 2); // Admin or above
                self::$configuration->setDefault('federation.security.method_permissions.update_client_name', 2); // Admin or above

                /** TamerLib Configuration */
                self::$configuration->setDefault('federation.tamer_lib.cli_workers', 8);
                self::$configuration->setDefault('federation.tamer_lib.node_workers', 20);
                self::$configuration->setDefault('federation.tamer_lib.server.host', '127.0.0.1');
                self::$configuration->setDefault('federation.tamer_lib.server.port', 6379);
                self::$configuration->setDefault('federation.tamer_lib.server.password', null);
                self::$configuration->setDefault('federation.tamer_lib.server.database', null);

                /** Save the configuration's default values if they don't exist */
                try
                {
                    self::$configuration->save();
                }
                catch(Exception $e)
                {
                    throw new RuntimeException('Failed to save configuration: ' . $e->getMessage(), $e);
                }
            }

            return self::$configuration->getConfiguration();
        }

        /**
         * Returns the configuration object.
         *
         * @return \ConfigLib\Configuration
         */
        public static function getConfigurationObject(): \ConfigLib\Configuration
        {
            if(self::$configuration === null)
            {
                self::getConfiguration();
            }

            return self::$configuration;
        }

        /**
         * @return TamerLibConfiguration
         */
        public static function getTamerLibConfiguration(): TamerLibConfiguration
        {
            if(self::$tamerlib_configuration === null)
            {
                self::$tamerlib_configuration = new TamerLibConfiguration(self::getConfiguration());
            }

            return self::$tamerlib_configuration;
        }

        /**
         * Returns driver of the database.
         *
         * @return string
         */
        public static function getDatabaseDriver(): string
        {
            return self::getConfiguration()['database']['driver'];
        }

        /**
         * Returns the host of the database.
         *
         * @return string
         */
        public static function getDatabaseHost(): string
        {
            return self::getConfiguration()['database']['host'];
        }

        /**
         * Returns the port of the database.
         *
         * @return int
         */
        public static function getDatabasePort(): int
        {
            return (int)self::getConfiguration()['database']['port'];
        }

        /**
         * Returns the name of the database.
         *
         * @return string
         */
        public static function getDatabaseName(): string
        {
            return self::getConfiguration()['database']['name'];
        }

        /**
         * Returns the username of the database.
         *
         * @return string
         */
        public static function getDatabaseUsername(): string
        {
            return self::getConfiguration()['database']['username'];
        }

        /**
         * Returns the password of the database.
         *
         * @return string|null
         */
        public static function getDatabasePassword(): ?string
        {
            $password = self::getConfiguration()['database']['password'];

            if($password === '')
            {
                return null;
            }

            return $password;
        }

        /**
         * Returns the charset of the database.
         *
         * @return string
         */
        public static function getDatabaseCharset(): string
        {
            return self::getConfiguration()['database']['charset'];
        }

        /**
         * Returns the interval in seconds to reconnect to the database.
         *
         * @return int
         */
        public static function getDatabaseReconnectInterval(): int
        {
            return (int)self::getConfiguration()['database']['reconnect_interval'];
        }

        /**
         * Returns the hostname of the server.
         *
         * @return string
         */
        public static function getHostName(): string
        {
            return self::getConfiguration()['federation']['hostname'];
        }

        /**
         * Returns True if the strict permission system is enabled and False if not.
         *
         * @return bool
         */
        public static function strictPermissionEnabled(): bool
        {
            return (bool)self::getConfiguration()['federation']['security']['strict_permissions'];
        }

        /**
         * Returns the permission level required to execute a command.
         *
         * @param string $command
         * @return int
         */
        public static function getMethodPermission(string $command): int
        {
            if(isset(self::getConfiguration()['federation']['security']['method_permissions'][$command]))
            {
                return (int)self::getConfiguration()['federation']['security']['method_permissions'][$command];
            }

            return match ($command)
            {
                Methods::CREATE_CLIENT => PermissionRole::ADMIN,
                default => PermissionRole::GUEST,
            };
        }

        /**
         * Returns True if the cache system is enabled for FederationLib
         * and False if not, based on the configuration.
         *
         * @return bool
         */
        public static function isCacheSystemEnabled(): bool
        {
            return (bool)self::getConfiguration()['cache_system']['enabled'];
        }

        /**
         * Returns the configuration for all the cache servers
         *
         * @return CacheServerConfiguration[]
         */
        public static function getCacheServers(): array
        {
            $results = [];
            foreach(self::getConfiguration()['cache_system']['servers'] as $server_name => $server)
            {
                if($server['enabled'] === true)
                {
                    $results[] = new CacheServerConfiguration($server_name, $server);
                }
            }

            return $results;
        }

        /**
         * Returns the additional priority to add/remove if the connection is opened.
         *
         * @return int
         */
        public static function getCacheOpenedConnectionPriority(): int
        {
            return (int)self::getConfiguration()['cache_system']['opened_connection_priority'];
        }

        /**
         * Returns additional priority to add/remove if the connection is closed.
         *
         * @return int
         */
        public static function getCacheErrorConnectionPriority(): int
        {
            return (int)self::getConfiguration()['cache_system']['error_connection_priority'];
        }

        /**
         * @param string $name
         * @return bool
         */
        public static function getObjectCacheEnabled(string $name): bool
        {
            return (bool)self::getConfiguration()['cache_system']['cache'][sprintf('%s_enabled', $name)];
        }

        /**
         * @param string $name
         * @return int
         */
        public static function getObjectCacheTtl(string $name): int
        {
            return (int)self::getConfiguration()['cache_system']['cache'][sprintf('%s_ttl', $name)];
        }

        /**
         * @param string $name
         * @return string
         */
        public static function getObjectCacheServerPreference(string $name): string
        {
            return self::getConfiguration()['cache_system']['cache'][sprintf('%s_server_preference', $name)];
        }

        /**
         * @param string $name
         * @return string
         */
        public static function getObjectCacheServerFallback(string $name): string
        {
            return self::getConfiguration()['cache_system']['cache'][sprintf('%s_server_fallback', $name)];
        }
    }