<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Classes;

    use Doctrine\DBAL\Connection;
    use Doctrine\DBAL\DriverManager;
    use Doctrine\DBAL\Logging\Middleware;
    use Exception;
    use FederationLib\Exceptions\DatabaseException;
    use LogLib\Log;
    use LogLib\Psr;

    class Database
    {
        /**
         * @var int|null
         */
        private static $sql_last_connection_time;

        /**
         * @var Connection|null
         */
        private static $sql_connection;

        /**
         * Returns/Establishes a connection to the database.
         *
         * @return Connection
         * @throws DatabaseException
         */
        public static function getConnection(): Connection
        {
            if(self::$sql_connection === null)
            {
                try
                {
                    Log::debug('net.nosial.federationlib', sprintf('Connecting to the database: %s://%s@%s:%s/%s', Configuration::getDatabaseDriver(), Configuration::getDatabaseUsername(), Configuration::getDatabaseHost(), Configuration::getDatabasePort(), Configuration::getDatabaseName()));
                    $connection = DriverManager::getConnection([
                        'driver' => Configuration::getDatabaseDriver(),
                        'host' => Configuration::getDatabaseHost(),
                        'port' => Configuration::getDatabasePort(),
                        'dbname' => Configuration::getDatabaseName(),
                        'user' => Configuration::getDatabaseUsername(),
                        'password' => Configuration::getDatabasePassword(),
                        'charset' => Configuration::getDatabaseCharset()
                    ]);

                    $connection->getConfiguration()->setMiddlewares([new Middleware(new Psr('com.dbal.doctrine'))]);
                    $connection->connect();
                }
                catch(Exception $e)
                {
                    throw new DatabaseException('Failed to connect to the database: ' . $e->getMessage(), $e);
                }

                self::$sql_connection = $connection;
                self::$sql_last_connection_time = time();
            }
            else
            {
                /** @noinspection NestedPositiveIfStatementsInspection */
                if(time() - self::$sql_last_connection_time > Configuration::getDatabaseReconnectInterval())
                {
                    Log::debug('net.nosial.federationlib', sprintf('Interval to reconnect to the %s server has been reached, reconnecting...', Configuration::getDatabaseDriver()));

                    // Reconnect to the database.
                    self::$sql_connection->close();
                    self::$sql_connection = null;

                    return self::getConnection();
                }
            }

            return self::$sql_connection;
        }

        /**
         * Returns the time of the last connection to the database. (null if no connection has been made)
         *
         * @return int|null
         */
        public static function getSqlLastConnectionTime(): ?int
        {
            return self::$sql_last_connection_time;
        }
    }